#!/usr/bin/env python

# Copyright (c) 2007, 2013 J. Lewis Muir <jlmuir@imca-cat.org>
#
# Permission to use, copy, modify, and distribute this software for any
# purpose with or without fee is hereby granted, provided that the above
# copyright notice and this permission notice appear in all copies.
#
# THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
# WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
# MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
# ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
# WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
# ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
# OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.

#
# For usage, see printUsage function or run with -h option.
#

import getopt
import asynchat
import asyncore
import os
import signal
import socket
import sys
import time

PORT = 50432
IS_DEBUG_OUTPUT_ENABLED = False

def main(args):
  parseCommandLineOptions(args)
  ConnectionDispatcher(PORT)
  asyncore.loop()

def getProgramName(args=None):
  if args == None: args = sys.argv
  if len(args) == 0 or args[0] == "-c": return "PROGRAM_NAME"
  return os.path.basename(args[0])

def initializeSignalHandlers():
  signalToNameMap = {
    signal.SIGHUP:"SIGHUP",
    signal.SIGINT:"SIGINT",
    signal.SIGPIPE:"SIGPIPE",
    signal.SIGALRM:"SIGALRM",
    signal.SIGTERM:"SIGTERM",
    signal.SIGUSR1:"SIGUSR1",
    signal.SIGUSR2:"SIGUSR2"
  }

  def signalHandler(signum, frame):
    if signum in [signal.SIGINT, signal.SIGTERM]:
      print
      print "%s caught. Exiting." % signalToNameMap[signum]
      sys.exit(0)
    else:
      print >> sys.stderr
      print >> sys.stderr, "%s caught. Exiting." % signalToNameMap[signum]
      sys.exit(1)

  for each in signalToNameMap.keys():
    signal.signal(each, signalHandler)

def parseCommandLineOptions(args):
  (options, extra) = getopt.getopt(args[1:], "dp:h", ["debug", "port=",
    "help"])

  if len(extra) > 0:
    print >> sys.stderr, "Error: unexpected command line option \"%s\"" % \
      extra[0]
    printUsage(sys.stderr)
    sys.exit(1)

  for eachOptName, eachOptValue in options:
    if eachOptName in ("-d", "--debug"):
      global IS_DEBUG_OUTPUT_ENABLED
      IS_DEBUG_OUTPUT_ENABLED = True
    elif eachOptName in ("-p", "--port"):
      global PORT
      PORT = int(eachOptValue)
    elif eachOptName in ("-h", "--help"):
      printUsage(sys.stdout)
      sys.exit(0)

def printUsage(outStream):
  print >> outStream, """\
Usage: %s [OPTION]...
Options:
  -d,--debug                Print debug messages to stderr
  -p,--port=NUMBER          Listen on the specified port NUMBER for incoming
                            connections (default: %d)
  -h,--help                 Print usage message and exit
""" % (getProgramName(), PORT)

class ConnectionDispatcher(asyncore.dispatcher):
  def __init__(self, port):
    asyncore.dispatcher.__init__ (self)
    self.port = port
    self.create_socket(socket.AF_INET, socket.SOCK_STREAM)
    self.set_reuse_addr()
    self.bind(("", port))
    self.listen(5)

  def handle_accept(self):
    ConnectionSession(self.accept())

class ConnectionSession(asynchat.async_chat):
  def __init__(self, (conn, addr)):
    asynchat.async_chat.__init__(self, conn)
    self.set_terminator("\r\n")
    self.outputTerminator = "\r\n"
    self.buffer = ""
    self.level = 50.0
    self.invalidLevel = None
    self.confHi = 90.0
    self.confLo = 20.0
    self.confA = 75.0
    self.confB = 45.0

  def collect_incoming_data(self, data):
    self.buffer = self.buffer + data

  def found_terminator(self):
    data = self.buffer
    self.buffer = ""
    self.handleClientRequest(data)

  def handleClientRequest(self, request):
    request = request.strip()
    if IS_DEBUG_OUTPUT_ENABLED: sys.stderr.write("< \"%s\"\n" % request)
    if request == "HI":
      self.sendClientResponse("%06.1f" % self.confHi)
    elif request.startswith("HI="):
      self.confHi = float(request[3:])
      self.sendClientResponse()
    elif request == "LO":
      self.sendClientResponse("%06.1f" % self.confLo)
    elif request.startswith("LO="):
      self.confLo = float(request[3:])
      self.sendClientResponse()
    elif request == "A":
      self.sendClientResponse("%06.1f" % self.confA)
    elif request.startswith("A="):
      self.confA = float(request[2:])
      self.sendClientResponse()
    elif request == "B":
      self.sendClientResponse("%06.1f" % self.confB)
    elif request.startswith("B="):
      self.confB = float(request[2:])
      self.sendClientResponse()
    elif request == "LEVEL":
      if self.invalidLevel != None:
        self.sendClientResponse(self.invalidLevel)
      else:
        self.sendClientResponse("%06.1f" % self.level)
    elif request.startswith("LEVEL="):
      # internal simulation-only command
      self.level = float(request[len("LEVEL="):])
      self.sendClientResponse()
    elif request.startswith("INVALID_LEVEL="):
      # internal simulation-only command
      self.invalidLevel = request[len("INVALID_LEVEL="):]
      if self.invalidLevel == "nil":
        self.invalidLevel = None
      self.sendClientResponse()
    else:
      self.sendClientResponse("-8")

  def sendClientResponse(self, response=""):
    self.push(response + self.outputTerminator)
    if IS_DEBUG_OUTPUT_ENABLED: sys.stderr.write("> \"%s\"\n" % response)

if __name__ == '__main__':
  try:
    initializeSignalHandlers()
    main(sys.argv)
  except Exception, e:
    if isinstance(e, SystemExit):
      raise e
    else:
      print >> sys.stderr
      print >> sys.stderr, "Error: %s" % e
      sys.exit(1)
